@@ -1,5 +1,5 @@
 /*
- * @copyright Copyright (c) OX Software GmbH, Germany <info@open-xchange.com>
+ * @copyright Copyright (c) Open-Xchange GmbH, Germany <info@open-xchange.com>
  * @license AGPL-3.0
  *
  * This code is free software: you can redistribute it and/or modify
@@ -80,14 +80,8 @@ public final class POP3MessageStorage extends MailMessageStorage implements ISim
 
     private MailAccount getMailAccount() throws OXException {
         if (mailAccount == null) {
-            try {
-                final MailAccountStorageService storageService = POP3ServiceRegistry.getServiceRegistry().getService(
-                    MailAccountStorageService.class,
-                    true);
-                mailAccount = storageService.getMailAccount(accountId, session.getUserId(), session.getContextId());
-            } catch (OXException e) {
-                throw e;
-            }
+            final MailAccountStorageService storageService = POP3ServiceRegistry.getServiceRegistry().getService(MailAccountStorageService.class, true);
+            mailAccount = storageService.getMailAccount(accountId, session.getUserId(), session.getContextId());
         }
         return mailAccount;
     }
