@@ -1,5 +1,5 @@
 /*
- * @copyright Copyright (c) OX Software GmbH, Germany <info@open-xchange.com>
+ * @copyright Copyright (c) Open-Xchange GmbH, Germany <info@open-xchange.com>
  * @license AGPL-3.0
  *
  * This code is free software: you can redistribute it and/or modify
@@ -54,8 +54,11 @@ import com.openexchange.mail.MailExceptionCode;
 import com.openexchange.mail.mime.MimeMailException;
 import com.openexchange.mail.mime.MimeMailExceptionCode;
 import com.openexchange.mail.mime.MimeSessionPropertyNames;
-import com.openexchange.mailaccount.MailAccount;
+import com.openexchange.mailaccount.Account;
 import com.openexchange.mailaccount.MailAccountStorageService;
+import com.openexchange.mailaccount.MailAccounts;
+import com.openexchange.mailaccount.utils.MailAccountUtils;
+import com.openexchange.net.HostList;
 import com.openexchange.net.ssl.SSLSocketFactoryProvider;
 import com.openexchange.net.ssl.config.SSLConfigurationService;
 import com.openexchange.net.ssl.exception.SSLExceptionCode;
@@ -238,13 +241,7 @@ public final class POP3StoreConnector {
             final IPOP3Properties pop3ConfProps = (IPOP3Properties) pop3Config.getMailProperties();
             final String server = pop3Config.getServer();
             final int port = pop3Config.getPort();
-            String staticCapabilities;
-            try {
-                staticCapabilities = POP3CapabilityCache.getCapability(InetAddress.getByName(IDNA.toASCII(server)), port, pop3Config.isSecure(), pop3ConfProps, pop3Config.getLogin());
-            } catch (Exception e) {
-                LOG.warn("Couldn't detect capabilities from POP3 server \"{}\" with login \"{}\" (user={}, context={})", server, pop3Config.getLogin(), I(session.getUserId()), I(session.getContextId()), e);
-                staticCapabilities = POP3CapabilityCache.getDeaultCapabilities();
-            }
+            String staticCapabilities = getCapabilities(pop3Config, session, pop3ConfProps);
             /*
              * JavaMail POP3 implementation requires capabilities "UIDL" and "TOP"
              */
@@ -274,6 +271,15 @@ public final class POP3StoreConnector {
             if (connectionTimeout > 0) {
                 pop3Props.put("mail.pop3.connectiontimeout", String.valueOf(connectionTimeout));
             }
+            /*
+             * Blocked hosts
+             */
+            if (pop3Config.getAccountId() != Account.DEFAULT_ID && !MailAccounts.isSecondaryAccount(pop3Config.getAccountId(), pop3Config.getSession().getUserId(), pop3Config.getSession().getContextId())) {
+                HostList blockedHosts = MailAccountUtils.getBlockedHosts();
+                if (!blockedHosts.isEmpty()) {
+                    pop3Props.put("mail.pop3.blockedHosts", blockedHosts);
+                }
+            }
             /*
              * Check if a secure POP3 connection should be established.
              *
@@ -384,7 +390,7 @@ public final class POP3StoreConnector {
                 // Log connect
                 AuditLogService auditLogService = getServiceRegistry().getOptionalService(AuditLogService.class);
                 if (null != auditLogService) {
-                    String eventId = MailAccount.DEFAULT_ID == accountId ? "pop3.primary.login" : "pop3.external.login";
+                    String eventId = Account.DEFAULT_ID == accountId ? "pop3.primary.login" : "pop3.external.login";
                     auditLogService.log(eventId, DefaultAttribute.valueFor(Name.LOGIN, session.getLoginName()), DefaultAttribute.valueFor(Name.IP_ADDRESS, session.getLocalIp()), DefaultAttribute.timestampFor(new Date()), DefaultAttribute.arbitraryFor("pop3.login", pop3Config.getLogin()), DefaultAttribute.arbitraryFor("pop3.server", server), DefaultAttribute.arbitraryFor("pop3.port", Integer.toString(port)));
                 }
 
@@ -424,7 +430,7 @@ public final class POP3StoreConnector {
                     }
                     throw POP3ExceptionCode.LOGIN_DELAY2.create(e, server, pop3Config.getLogin(), Integer.valueOf(session.getUserId()), Integer.valueOf(session.getContextId()), Integer.valueOf(seconds), e.getMessage());
                 }
-                if (accountId != MailAccount.DEFAULT_ID) {
+                if (accountId != Account.DEFAULT_ID) {
                     AbstractTask<Void> task = new AbstractTask<Void>() {
 
                         @Override
@@ -441,17 +447,20 @@ public final class POP3StoreConnector {
                 throw e;
             } catch (MessagingException e) {
                 if (MimeMailException.isSSLHandshakeException(e)) {
-                    List<Object> displayArgs = new ArrayList<>(2);
-                    displayArgs.add(SSLExceptionCode.extractArgument(e, "fingerprint"));
-                    displayArgs.add(server);
-                    throw SSLExceptionCode.UNTRUSTED_CERTIFICATE.create(e.getCause(), displayArgs.toArray(new Object[] {}));
+                    String fingerPrint = SSLExceptionCode.extractArgument(e, "fingerprint");
+                    if (Strings.isNotEmpty(fingerPrint)) {
+                        List<Object> displayArgs = new ArrayList<>(2);
+                        displayArgs.add(fingerPrint);
+                        displayArgs.add(server);
+                        throw SSLExceptionCode.UNTRUSTED_CERTIFICATE.create(e.getCause(), displayArgs.toArray(new Object[] {}));
+                    }
                 }
 
                 final Exception nested = e.getNextException();
                 if (nested != null) {
                     if (nested instanceof IOException) {
                         throw MimeMailExceptionCode.CONNECT_ERROR.create(e, pop3Config.getServer(), pop3Config.getLogin());
-                    } else if (tmpDownEnabled && SocketTimeoutException.class.isInstance(e.getNextException())) {
+                    } else if (tmpDownEnabled && e.getNextException() instanceof SocketTimeoutException) {
                         /*
                          * TODO: Re-think if exception's message should be part of condition or just checking if nested exception is an instance of
                          * SocketTimeoutException
@@ -518,6 +527,17 @@ public final class POP3StoreConnector {
         }
     }
 
+    private static String getCapabilities(POP3Config pop3Config, final Session session, final IPOP3Properties pop3ConfProps) {
+        String staticCapabilities;
+        try {
+            staticCapabilities = POP3CapabilityCache.getCapability(InetAddress.getByName(IDNA.toASCII(pop3Config.getServer())), pop3Config.getPort(), pop3Config.isSecure(), pop3ConfProps, pop3Config.getLogin());
+        } catch (Exception e) {
+            LOG.warn("Couldn't detect capabilities from POP3 server \"{}\" with login \"{}\" (user={}, context={})", pop3Config.getServer(), pop3Config.getLogin(), I(session.getUserId()), I(session.getContextId()), e);
+            staticCapabilities = POP3CapabilityCache.getDeaultCapabilities();
+        }
+        return staticCapabilities;
+    }
+
     private static void checkFailedAuths(final String login, final String pass) throws AuthenticationFailedException {
         final LoginAndPass key = new LoginAndPass(login, pass);
         final Long range = failedAuths.get(key);
