@@ -1,5 +1,5 @@
 /*
- * @copyright Copyright (c) OX Software GmbH, Germany <info@open-xchange.com>
+ * @copyright Copyright (c) Open-Xchange GmbH, Germany <info@open-xchange.com>
  * @license AGPL-3.0
  *
  * This code is free software: you can redistribute it and/or modify
@@ -145,7 +145,7 @@ public final class POP3SyncMessagesCallable implements Callable<Object> {
         return null;
     }
 
-    private boolean isConnectable(final long refreshRateMillis, final Long lastAccessed) {
+    private static boolean isConnectable(final long refreshRateMillis, final Long lastAccessed) {
         return ((null == lastAccessed) || ((System.currentTimeMillis() - lastAccessed.longValue()) >= refreshRateMillis));
     }
 
