@@ -1,5 +1,5 @@
 /*
- * @copyright Copyright (c) OX Software GmbH, Germany <info@open-xchange.com>
+ * @copyright Copyright (c) Open-Xchange GmbH, Germany <info@open-xchange.com>
  * @license AGPL-3.0
  *
  * This code is free software: you can redistribute it and/or modify
@@ -29,6 +29,7 @@ import com.openexchange.caching.CacheService;
 import com.openexchange.capabilities.CapabilityService;
 import com.openexchange.charset.CharsetService;
 import com.openexchange.config.ConfigurationService;
+import com.openexchange.config.Reloadable;
 import com.openexchange.config.cascade.ConfigViewFactory;
 import com.openexchange.context.ContextService;
 import com.openexchange.groupware.settings.PreferencesItemService;
@@ -42,6 +43,7 @@ import com.openexchange.osgi.HousekeepingActivator;
 import com.openexchange.osgi.ServiceRegistry;
 import com.openexchange.pop3.Enabled;
 import com.openexchange.pop3.POP3Provider;
+import com.openexchange.pop3.config.POP3Reloadable;
 import com.openexchange.pop3.storage.POP3StorageProvider;
 import com.openexchange.pop3.storage.mailaccount.MailAccountPOP3StorageProvider;
 import com.openexchange.sessiond.SessiondService;
@@ -132,6 +134,10 @@ public final class POP3Activator extends HousekeepingActivator {
              * Register
              */
             registerService(PreferencesItemService.class, new Enabled(getService(ConfigViewFactory.class)), null);
+            /*
+             * Register reloadable
+             */
+            registerService(Reloadable.class, POP3Reloadable.getInstance());
         } catch (Exception e) {
             LOG.error("", e);
             throw e;
