@@ -1,5 +1,5 @@
 /*
- * @copyright Copyright (c) OX Software GmbH, Germany <info@open-xchange.com>
+ * @copyright Copyright (c) Open-Xchange GmbH, Germany <info@open-xchange.com>
  * @license AGPL-3.0
  *
  * This code is free software: you can redistribute it and/or modify
@@ -32,13 +32,13 @@ import com.openexchange.mail.api.IMailMessageStorage;
  * @author <a href="mailto:thorben.betten@open-xchange.com">Thorben Betten</a>
  */
 public interface POP3Storage {
-    
+
     /**
      * Connects this POP3 storage.
      *
      * @throws OXException If establishing a connection for this storage fails
      */
-    default public void connect() throws OXException {
+    public default void connect() throws OXException {
         connect(false);
     }
 
