@@ -1,5 +1,5 @@
 /*
- * @copyright Copyright (c) OX Software GmbH, Germany <info@open-xchange.com>
+ * @copyright Copyright (c) Open-Xchange GmbH, Germany <info@open-xchange.com>
  * @license AGPL-3.0
  *
  * This code is free software: you can redistribute it and/or modify
@@ -46,7 +46,7 @@ public interface POP3StorageTrashContainer {
      * @param uidl The UIDLs of the permanently deleted POP3 messages
      * @throws OXException If adding UIDLs fails
      */
-    public void addAllUIDL(Collection<? extends String> uidls) throws OXException;
+    public void addAllUIDL(Collection<String> uidls) throws OXException;
 
     /**
      * Removes specified UIDL from this trash container.
