@@ -1,5 +1,5 @@
 /*
- * @copyright Copyright (c) OX Software GmbH, Germany <info@open-xchange.com>
+ * @copyright Copyright (c) Open-Xchange GmbH, Germany <info@open-xchange.com>
  * @license AGPL-3.0
  *
  * This code is free software: you can redistribute it and/or modify
@@ -27,6 +27,7 @@ import java.util.ArrayList;
 import java.util.List;
 import java.util.concurrent.Callable;
 import java.util.concurrent.locks.Lock;
+import javax.mail.Store;
 import com.openexchange.context.ContextService;
 import com.openexchange.exception.OXException;
 import com.openexchange.groupware.contexts.Context;
@@ -39,8 +40,12 @@ import com.openexchange.mail.OrderDirection;
 import com.openexchange.mail.Quota;
 import com.openexchange.mail.Quota.Type;
 import com.openexchange.mail.api.IMailFolderStorage;
+import com.openexchange.mail.api.IMailFolderStorageEnhanced;
+import com.openexchange.mail.api.IMailFolderStorageEnhanced2;
 import com.openexchange.mail.api.IMailFolderStorageInfoSupport;
+import com.openexchange.mail.api.IMailFolderStorageSubscription;
 import com.openexchange.mail.api.IMailFolderStorageTrashAware;
+import com.openexchange.mail.api.IMailStoreAware;
 import com.openexchange.mail.dataobjects.MailFolder;
 import com.openexchange.mail.dataobjects.MailFolder.DefaultFolderType;
 import com.openexchange.mail.dataobjects.MailFolderDescription;
@@ -69,7 +74,7 @@ import com.openexchange.tools.session.ServerSession;
  *
  * @author <a href="mailto:thorben.betten@open-xchange.com">Thorben Betten</a>
  */
-public final class MailAccountPOP3FolderStorage implements IMailFolderStorage, IMailFolderStorageInfoSupport, IMailFolderStorageTrashAware {
+public final class MailAccountPOP3FolderStorage implements IMailFolderStorage, IMailFolderStorageEnhanced2, IMailFolderStorageInfoSupport, IMailFolderStorageTrashAware, IMailFolderStorageSubscription {
 
     static final org.slf4j.Logger LOG =
         org.slf4j.LoggerFactory.getLogger(MailAccountPOP3FolderStorage.class);
@@ -135,11 +140,7 @@ public final class MailAccountPOP3FolderStorage implements IMailFolderStorage, I
                     if (session instanceof ServerSession) {
                         this.ctx = ctx = ((ServerSession) session).getContext();
                     } else {
-                        try {
-                            this.ctx = ctx = POP3ServiceRegistry.getServiceRegistry().getService(ContextService.class, true).getContext(session.getContextId());
-                        } catch (OXException e) {
-                            throw e;
-                        }
+                        this.ctx = ctx = POP3ServiceRegistry.getServiceRegistry().getService(ContextService.class, true).getContext(session.getContextId());
                     }
                 }
             } // synchronized
@@ -197,7 +198,7 @@ public final class MailAccountPOP3FolderStorage implements IMailFolderStorage, I
         mailFolder.setDefaultFolderType(DefaultFolderType.NONE);
     }
 
-    private <V> V performSynchronized(final Callable<V> task, final Session session) throws Exception {
+    private static <V> V performSynchronized(final Callable<V> task, final Session session) throws Exception {
         Lock lock = (Lock) session.getParameter(Session.PARAM_LOCK);
         if (null == lock) {
             lock = Session.EMPTY_LOCK;
@@ -212,7 +213,127 @@ public final class MailAccountPOP3FolderStorage implements IMailFolderStorage, I
 
     @Override
     public <T> T supports(Class<T> iface) throws OXException {
-        return delegatee.supports(iface);
+        T instance = delegatee.supports(iface);
+        if (instance == null) {
+            // Not supported by backing folder storage
+            return null;
+        }
+        if (IMailFolderStorageEnhanced2.class.equals(iface)) {
+            return (T) this;
+        }
+        if (IMailFolderStorageEnhanced.class.equals(iface)) {
+            return (T) this;
+        }
+        if (IMailFolderStorageInfoSupport.class.equals(iface)) {
+            return (T) this;
+        }
+        if (IMailFolderStorageTrashAware.class.equals(iface)) {
+            return (T) this;
+        }
+        if (IMailFolderStorageSubscription.class.equals(iface)) {
+            return (T) this;
+        }
+        return instance;
+    }
+
+    @Override
+    public int[] getTotalAndUnreadCounter(String fullName) throws OXException {
+        IMailFolderStorageEnhanced2 folderStorage = delegatee.supports(IMailFolderStorageEnhanced2.class);
+        if (folderStorage == null) {
+            throw MailExceptionCode.UNSUPPORTED_OPERATION.create();
+        }
+
+        if (MailFolder.ROOT_FOLDER_ID.equals(fullName)) {
+            return new int[] { 0, 0 };
+        }
+
+        return folderStorage.getTotalAndUnreadCounter(getRealFullname(fullName));
+    }
+
+    @Override
+    public void expungeFolder(String fullName) throws OXException {
+        IMailFolderStorageEnhanced folderStorage = delegatee.supports(IMailFolderStorageEnhanced.class);
+        if (folderStorage == null) {
+            throw MailExceptionCode.UNSUPPORTED_OPERATION.create();
+        }
+
+        if (MailFolder.ROOT_FOLDER_ID.equals(fullName)) {
+            return;
+        }
+
+        folderStorage.expungeFolder(getRealFullname(fullName));
+    }
+
+    @Override
+    public void expungeFolder(String fullName, boolean hardDelete) throws OXException {
+        IMailFolderStorageEnhanced folderStorage = delegatee.supports(IMailFolderStorageEnhanced.class);
+        if (folderStorage == null) {
+            throw MailExceptionCode.UNSUPPORTED_OPERATION.create();
+        }
+
+        if (MailFolder.ROOT_FOLDER_ID.equals(fullName)) {
+            return;
+        }
+
+        folderStorage.expungeFolder(getRealFullname(fullName), hardDelete);
+    }
+
+    @Override
+    public int getUnreadCounter(String fullName) throws OXException {
+        IMailFolderStorageEnhanced folderStorage = delegatee.supports(IMailFolderStorageEnhanced.class);
+        if (folderStorage == null) {
+            throw MailExceptionCode.UNSUPPORTED_OPERATION.create();
+        }
+
+        if (MailFolder.ROOT_FOLDER_ID.equals(fullName)) {
+            return 0;
+        }
+
+        return folderStorage.getUnreadCounter(getRealFullname(fullName));
+    }
+
+    @Override
+    public int getNewCounter(String fullName) throws OXException {
+        IMailFolderStorageEnhanced folderStorage = delegatee.supports(IMailFolderStorageEnhanced.class);
+        if (folderStorage == null) {
+            throw MailExceptionCode.UNSUPPORTED_OPERATION.create();
+        }
+
+        if (MailFolder.ROOT_FOLDER_ID.equals(fullName)) {
+            return 0;
+        }
+
+        return folderStorage.getNewCounter(getRealFullname(fullName));
+    }
+
+    @Override
+    public int getTotalCounter(String fullName) throws OXException {
+        IMailFolderStorageEnhanced folderStorage = delegatee.supports(IMailFolderStorageEnhanced.class);
+        if (folderStorage == null) {
+            throw MailExceptionCode.UNSUPPORTED_OPERATION.create();
+        }
+
+        if (MailFolder.ROOT_FOLDER_ID.equals(fullName)) {
+            return 0;
+        }
+
+        return folderStorage.getTotalCounter(getRealFullname(fullName));
+    }
+
+    @Override
+    public boolean isSubscribeSupported() throws OXException {
+        IMailFolderStorageSubscription storageSubscription = delegatee.supports(IMailFolderStorageSubscription.class);
+        return null != storageSubscription && storageSubscription.isSubscribeSupported();
+    }
+
+    @Override
+    public void subscribe(String fullName, boolean subscribe, boolean errorOnFailure) throws OXException {
+        IMailFolderStorageSubscription storageSubscription = delegatee.supports(IMailFolderStorageSubscription.class);
+        if (null != storageSubscription && storageSubscription.isSubscribeSupported()) {
+            String realFullname = getRealFullname(fullName);
+            storageSubscription.subscribe(realFullname, subscribe, errorOnFailure);
+        }
+        throw MailExceptionCode.UNSUPPORTED_OPERATION.create();
     }
 
     @Override
@@ -263,20 +384,13 @@ public final class MailAccountPOP3FolderStorage implements IMailFolderStorage, I
                     /*
                      * Load mail account
                      */
-                    final boolean isSpamOptionEnabled;
-                    final MailAccount mailAccount;
-                    try {
-                        mailAccount =
-                            POP3ServiceRegistry.getServiceRegistry().getService(MailAccountStorageService.class, true).getMailAccount(
-                                accountId,
-                                session.getUserId(),
-                                session.getContextId());
-                        final UserSettingMail usm =
-                            UserSettingMailStorage.getInstance().getUserSettingMail(session.getUserId(), getContext());
-                        isSpamOptionEnabled = usm.isSpamOptionEnabled();
-                    } catch (OXException e) {
-                        throw e;
-                    }
+                    final MailAccount mailAccount =
+                        POP3ServiceRegistry.getServiceRegistry().getService(MailAccountStorageService.class, true).getMailAccount(
+                            accountId,
+                            session.getUserId(),
+                            session.getContextId());
+                    final UserSettingMail usm = UserSettingMailStorage.getInstance().getUserSettingMail(session.getUserId(), getContext());
+                    boolean isSpamOptionEnabled = usm.isSpamOptionEnabled();
                     /*
                      * Get default folder names
                      */
@@ -408,7 +522,7 @@ public final class MailAccountPOP3FolderStorage implements IMailFolderStorage, I
             // Delegate delete to message storage which already keeps track of proper mappings
             getMessageStorage().deleteMessages(fullname, mailIDs, true);
             // Check for trash folder
-            if (getTrashFolder().equals(fullname)) {
+            if (fullname.equals(getTrashFolder())) {
                 final MailFolder[] trashSubfolders = getSubfolders(getTrashFolder(), true);
                 for (final MailFolder trashSubfolder : trashSubfolders) {
                     deleteFolder(trashSubfolder.getFullname(), true);
@@ -683,11 +797,7 @@ public final class MailAccountPOP3FolderStorage implements IMailFolderStorage, I
 
     @Override
     public String moveFolder(final String fullname, final String newFullname) throws OXException {
-        if (MailFolder.ROOT_FOLDER_ID.equals(newFullname)) {
-            throw POP3ExceptionCode.MOVE_ILLEGAL.create();
-        } else if (isDefaultFolder(fullname)) {
-            throw POP3ExceptionCode.MOVE_ILLEGAL.create();
-        } else if (isDefaultFolder(newFullname)) {
+        if (MailFolder.ROOT_FOLDER_ID.equals(newFullname) || isDefaultFolder(fullname) || isDefaultFolder(newFullname)) {
             throw POP3ExceptionCode.MOVE_ILLEGAL.create();
         }
         final String[] parentAndName = getParentAndName(newFullname, storage.getSeparator());
@@ -774,7 +884,7 @@ public final class MailAccountPOP3FolderStorage implements IMailFolderStorage, I
         return prependPath2Fullname(path, storage.getSeparator(), fullname);
     }
 
-    private String[] getParentAndName(final String fullname, final char separator) {
+    private static String[] getParentAndName(final String fullname, final char separator) {
         final int pos = fullname.lastIndexOf(separator);
         if (-1 == pos) {
             return new String[] { MailFolder.ROOT_FOLDER_ID, fullname };
@@ -782,11 +892,45 @@ public final class MailAccountPOP3FolderStorage implements IMailFolderStorage, I
         return new String[] { fullname.substring(0, pos), fullname.substring(pos + 1) };
     }
 
-    private void unsubscribe(final String realFullName) throws OXException {
-        final MailFolderDescription description = new MailFolderDescription();
-        description.setFullname(realFullName);
-        description.setSubscribed(false);
-        delegatee.updateFolder(realFullName, description);
+    private void unsubscribe(final String realFullName) {
+        try {
+            IMailFolderStorageSubscription storageSubscription = getStorageSubscriptionSafe();
+            if (storageSubscription != null) {
+                storageSubscription.subscribe(realFullName, false, false);
+            } else {
+                MailFolderDescription description = new MailFolderDescription();
+                description.setFullname(realFullName);
+                description.setSubscribed(false);
+                delegatee.updateFolder(realFullName, description);
+            }
+        } catch (Exception e) {
+            Store store = getStoreSafe();
+            if (store != null) {
+                LOG.warn("Failed to unsubscribe backing folder {} on {}", realFullName, store, e);
+            } else {
+                LOG.warn("Failed to unsubscribe backing folder {}", realFullName, e);
+            }
+        }
+    }
+
+    private IMailFolderStorageSubscription getStorageSubscriptionSafe() {
+        try {
+            IMailFolderStorageSubscription storageSubscription = delegatee.supports(IMailFolderStorageSubscription.class);
+            return null != storageSubscription && storageSubscription.isSubscribeSupported() ? storageSubscription : null;
+        } catch (Exception e) {
+            LOG.debug("Failed to retrieve mail folder storage's subscription extension", e);
+            return null;
+        }
+    }
+
+    private Store getStoreSafe() {
+        try {
+            IMailStoreAware storeAware = delegatee.supports(IMailStoreAware.class);
+            return null != storeAware && storeAware.isStoreSupported() ? storeAware.getStore() : null;
+        } catch (Exception e) {
+            LOG.debug("Failed to retrieve store", e);
+            return null;
+        }
     }
 
 }
