@@ -1,5 +1,5 @@
 /*
- * @copyright Copyright (c) OX Software GmbH, Germany <info@open-xchange.com>
+ * @copyright Copyright (c) Open-Xchange GmbH, Germany <info@open-xchange.com>
  * @license AGPL-3.0
  *
  * This code is free software: you can redistribute it and/or modify
@@ -84,14 +84,10 @@ public class MailAccountPOP3MessageStorage implements ISimplifiedThreadStructure
 
     private MailAccount getMailAccount() throws OXException {
         if (mailAccount == null) {
-            try {
-                final MailAccountStorageService storageService = POP3ServiceRegistry.getServiceRegistry().getService(
-                    MailAccountStorageService.class,
-                    true);
-                mailAccount = storageService.getMailAccount(pop3AccountId, session.getUserId(), session.getContextId());
-            } catch (OXException e) {
-                throw e;
-            }
+            final MailAccountStorageService storageService = POP3ServiceRegistry.getServiceRegistry().getService(
+                MailAccountStorageService.class,
+                true);
+            mailAccount = storageService.getMailAccount(pop3AccountId, session.getUserId(), session.getContextId());
         }
         return mailAccount;
     }
@@ -160,10 +156,7 @@ public class MailAccountPOP3MessageStorage implements ISimplifiedThreadStructure
             }
         }
         IMailMessageStorageMimeSupport streamSupport = delegatee.supports(IMailMessageStorageMimeSupport.class);
-        if (null == streamSupport) {
-            throw MailExceptionCode.UNSUPPORTED_OPERATION.create();
-        }
-        if (!streamSupport.isMimeSupported()) {
+        if (null == streamSupport || !streamSupport.isMimeSupported()) {
             throw MailExceptionCode.UNSUPPORTED_OPERATION.create();
         }
         String[] uids = streamSupport.appendMimeMessages(getRealFullname("INBOX"), pop3Messages);
@@ -396,7 +389,9 @@ public class MailAccountPOP3MessageStorage implements ISimplifiedThreadStructure
 
     @Override
     public MailMessage saveDraft(final String draftFullname, final ComposedMailMessage draftMail) throws OXException {
-        return delegatee.saveDraft(getRealFullname(draftFullname), draftMail);
+        MailMessage savedDraft = delegatee.saveDraft(getRealFullname(draftFullname), draftMail);
+        setFolderAndAccount(draftFullname, savedDraft);
+        return savedDraft;
     }
 
     @Override
