@@ -1,5 +1,5 @@
 /*
- * @copyright Copyright (c) OX Software GmbH, Germany <info@open-xchange.com>
+ * @copyright Copyright (c) Open-Xchange GmbH, Germany <info@open-xchange.com>
  * @license AGPL-3.0
  *
  * This code is free software: you can redistribute it and/or modify
@@ -25,17 +25,18 @@ import static com.openexchange.database.Databases.closeSQLStuff;
 import static com.openexchange.java.Autoboxing.I;
 import java.io.IOException;
 import java.lang.reflect.Field;
-import java.security.SecureRandom;
 import java.sql.Connection;
 import java.sql.PreparedStatement;
 import java.sql.ResultSet;
 import java.sql.SQLException;
 import java.util.ArrayList;
+import java.util.Arrays;
 import java.util.Collection;
 import java.util.Collections;
 import java.util.HashSet;
 import java.util.List;
 import java.util.Set;
+import java.util.concurrent.TimeUnit;
 import javax.mail.FetchProfile;
 import javax.mail.Flags;
 import javax.mail.Folder;
@@ -46,6 +47,8 @@ import javax.mail.UIDFolder;
 import javax.mail.internet.MimeMessage;
 import com.openexchange.databaseold.Database;
 import com.openexchange.exception.OXException;
+import com.openexchange.java.Strings;
+import com.openexchange.java.CryptoUtil;
 import com.openexchange.mail.MailExceptionCode;
 import com.openexchange.mail.MailServletInterface;
 import com.openexchange.mail.api.IMailFolderStorage;
@@ -74,7 +77,10 @@ import com.openexchange.pop3.storage.POP3StoragePropertyNames;
 import com.openexchange.pop3.storage.POP3StorageTrashContainer;
 import com.openexchange.pop3.storage.POP3StorageUIDLMap;
 import com.openexchange.pop3.storage.mailaccount.util.Utility;
+import com.openexchange.server.ServiceExceptionCode;
 import com.openexchange.session.Session;
+import com.openexchange.timer.ScheduledTimerTask;
+import com.openexchange.timer.TimerService;
 import com.sun.mail.pop3.POP3Folder;
 import com.sun.mail.pop3.POP3Message;
 import com.sun.mail.pop3.POP3Store;
@@ -89,7 +95,7 @@ import gnu.trove.map.hash.TIntObjectHashMap;
 public class MailAccountPOP3Storage implements POP3Storage, IMailStoreAware {
 
     private static final org.slf4j.Logger LOG = org.slf4j.LoggerFactory.getLogger(MailAccountPOP3Storage.class);
-
+    
     /*-
      * Member section
      */
@@ -110,7 +116,6 @@ public class MailAccountPOP3Storage implements POP3Storage, IMailStoreAware {
         this.pop3Access = pop3Access;
         pop3AccountId = pop3Access.getAccountId();
         final Session session = pop3Access.getSession();
-        // defaultMailAccess = MailAccess.getInstance(session);
         this.properties = properties;
         {
             String tmp = properties.getProperty(POP3StoragePropertyNames.PROPERTY_PATH);
@@ -152,15 +157,8 @@ public class MailAccountPOP3Storage implements POP3Storage, IMailStoreAware {
              * Check location of trash folder: beside or below INBOX folder?
              */
             final int pos = trashFullname.lastIndexOf(sep);
-            final String accountName;
-            try {
-                final MailAccountStorageService storageService = POP3ServiceRegistry.getServiceRegistry().getService(
-                    MailAccountStorageService.class,
-                    true);
-                accountName = stripSpecials(storageService.getMailAccount(pop3AccountId, user, cid).getName());
-            } catch (OXException e) {
-                throw e;
-            }
+            final MailAccountStorageService storageService = POP3ServiceRegistry.getServiceRegistry().getService(MailAccountStorageService.class, true);
+            final String accountName = stripSpecials(storageService.getMailAccount(pop3AccountId, user, cid).getName());
             String fullname;
             if (pos == -1) {
                 /*
@@ -178,9 +176,8 @@ public class MailAccountPOP3Storage implements POP3Storage, IMailStoreAware {
              */
             if (defaultMailAccess.getFolderStorage().exists(fullname)) {
                 final String pre = fullname;
-                final SecureRandom secureRandom = new SecureRandom();
                 do {
-                    fullname = pre + stripSpecials(String.valueOf(secureRandom.nextInt(1000)));
+                    fullname = pre + stripSpecials(String.valueOf(CryptoUtil.getSecureRandom().nextInt(1000)));
                 } while (defaultMailAccess.getFolderStorage().exists(fullname));
             }
             /*
@@ -404,10 +401,7 @@ public class MailAccountPOP3Storage implements POP3Storage, IMailStoreAware {
                 try {
                     fs.createFolder(toCreate);
                 } catch (OXException e) {
-                    throw POP3ExceptionCode.ILLEGAL_PATH.create(e,
-                        path,
-                        Integer.valueOf(session.getUserId()),
-                        Integer.valueOf(session.getContextId()));
+                    throw POP3ExceptionCode.ILLEGAL_PATH.create(e, path, Integer.valueOf(session.getUserId()), Integer.valueOf(session.getContextId()));
                 }
 
                 /*
@@ -509,34 +503,27 @@ public class MailAccountPOP3Storage implements POP3Storage, IMailStoreAware {
             try {
                 int pos = 1;
                 if (null == lastAccessed) {
-                    stmt = con.prepareStatement("INSERT INTO user_mail_account_properties (id,cid,user,name,value) VALUES (?,?,?,?,?)");
+                    stmt = con.prepareStatement("INSERT INTO `user_mail_account_properties` (`id`,`cid`,`user`,`name`,`value`) VALUES (?,?,?,?,?) ON DUPLICATE KEY UPDATE `cid`=`cid`");
                     stmt.setInt(pos++, accountId);
                     stmt.setInt(pos++, cid);
                     stmt.setInt(pos++, user);
                     stmt.setString(pos++, POP3StoragePropertyNames.PROPERTY_LAST_ACCESSED);
                     stmt.setString(pos++, "0"); // Dummy last-accessed
                     // Uniquely INSERT
-                    try {
-                        final int result = stmt.executeUpdate();
-                        if (result <= 0) {
-                            // Another thread is already in process.
-                            return;
-                        }
-                    } catch (SQLException e) {
-                        // INSERT failed. Another thread is already in process.
+                    if (stmt.executeUpdate() <= 0) {
+                        // Another thread is already processing
                         return;
                     }
                 } else {
-                    stmt = con.prepareStatement("UPDATE user_mail_account_properties SET value=? WHERE id=? AND cid=? AND user=? AND name=? AND value=?");
+                    stmt = con.prepareStatement("UPDATE `user_mail_account_properties` SET `value`=? WHERE `id`=? AND `cid`=? AND `user`=? AND `name`=? AND `value`=?");
                     stmt.setString(pos++, Long.toString(lastAccessed.longValue() + 1));
                     stmt.setInt(pos++, accountId);
                     stmt.setInt(pos++, cid);
                     stmt.setInt(pos++, user);
                     stmt.setString(pos++, POP3StoragePropertyNames.PROPERTY_LAST_ACCESSED);
                     stmt.setString(pos++, lastAccessed.toString());
-                    final int result = stmt.executeUpdate();
-                    if (result <= 0) {
-                        // Another thread is already in process.
+                    if (stmt.executeUpdate() <= 0) {
+                        // Another thread is already processing
                         return;
                     }
                 }
@@ -549,13 +536,14 @@ public class MailAccountPOP3Storage implements POP3Storage, IMailStoreAware {
             }
         }
         // Acquire DB lock
-        if (false == acquireLock(session)) {
-            // Another thread is already in process.
+        POP3Store pop3Store = null;
+        DatabaseLock lock = acquireLock(session);
+        if (lock == null) {
+            // Another thread is already processing
             throw new AlreadyLockedException();
         }
-        // Start sync process
-        POP3Store pop3Store = null;
         try {
+            // Locked. Start sync process.
             POP3Config pop3Config = pop3Access.getPOP3Config();
             boolean forceSecure = pop3AccountId > 0 && pop3Config.isRequireTls();
             final POP3StoreResult result = POP3StoreConnector.getPOP3Store(pop3Config, pop3Access.getMailProperties(), false, pop3AccountId, session, !expunge, forceSecure);
@@ -602,7 +590,7 @@ public class MailAccountPOP3Storage implements POP3Storage, IMailStoreAware {
                         final long startMillis = System.currentTimeMillis();
                         inbox.fetch(all, FETCH_PROFILE_UID);
                         MailServletInterface.mailInterfaceMonitor.addUseTime(System.currentTimeMillis() - startMillis);
-                        for (int i = 0; i < all.length; i++) {
+                        for (int i = all.length; i-- > 0;) {
                             final Message message = all[i];
                             final String uidl = inbox.getUID(message);
                             seqnum2uidl.put(message.getMessageNumber(), uidl);
@@ -618,10 +606,18 @@ public class MailAccountPOP3Storage implements POP3Storage, IMailStoreAware {
                         blockSize = configuredBlockSize > messageCount ? messageCount : configuredBlockSize;
                     }
                     int start = 1;
+                    Set<String> uidlsOfFailedMessages = null;
                     while (start <= messageCount) {
-                        final int num = add2Storage(inbox, start, blockSize, uidlNotSupported, messageCount, seqnum2uidl);
-                        start += num;
+                        Add2StorageResult addedResult = add2Storage(inbox, start, blockSize, uidlNotSupported, messageCount, seqnum2uidl);
+                        start += addedResult.numberOfAddedMessages;
                         clear(messageCache);
+                        if (addedResult.optionalUidlsOfFailedMessages != null) {
+                            if (uidlsOfFailedMessages == null) {
+                                uidlsOfFailedMessages = new HashSet<String>(addedResult.optionalUidlsOfFailedMessages);
+                            } else {
+                                uidlsOfFailedMessages.addAll(addedResult.optionalUidlsOfFailedMessages);
+                            }
+                        }
                     }
                     /*
                      * Expunge if necessary
@@ -631,8 +627,20 @@ public class MailAccountPOP3Storage implements POP3Storage, IMailStoreAware {
                          * Expunge all messages
                          */
                         final Message[] messages = inbox.getMessages();
-                        for (int i = 0; i < messages.length; i++) {
-                            messages[i].setFlags(FLAGS_DELETED, true);
+                        if (uidlsOfFailedMessages != null) {
+                            for (int i = 0; i < messages.length; i++) {
+                                Message message = messages[i];
+                                String uidl = seqnum2uidl.get(message.getMessageNumber());
+                                if (uidl != null && uidlsOfFailedMessages.contains(uidl)) {
+                                    LOG.debug("No expunge of non-synced POP3 message {} of account {} from user {} in cotnext {}", uidl, I(pop3Access.getAccountId()), I(session.getUserId()), I(session.getContextId()));
+                                } else {
+                                    message.setFlags(FLAGS_DELETED, true);
+                                }
+                            }
+                        } else {
+                            for (int i = 0; i < messages.length; i++) {
+                                messages[i].setFlags(FLAGS_DELETED, true);
+                            }
                         }
                         doExpunge = true;
                     } else if (isDeleteWriteThrough()) {
@@ -654,13 +662,7 @@ public class MailAccountPOP3Storage implements POP3Storage, IMailStoreAware {
                     }
                 }
             } finally {
-                try {
-                    if (inbox.isOpen()) {
-                        inbox.close(doExpunge);
-                    }
-                } catch (Exception e) {
-                    LOG.warn("POP3 mailbox {} could not be expunged/closed for login {}", pop3Config.getServer(), pop3Config.getLogin(), e);
-                }
+                close(inbox, doExpunge, pop3Config);
                 // Trashed UIDLs not needed anymore
                 if (doExpunge) {
                     getTrashContainer().clear();
@@ -686,98 +688,150 @@ public class MailAccountPOP3Storage implements POP3Storage, IMailStoreAware {
                 if (null != pop3Store) {
                     pop3Store.close();
                 }
-            } catch (MessagingException e) {
+            } catch (Exception e) {
                 LOG.error("", e);
             }
-            releaseLock(session);
+            lock.unlock();
         }
     }
 
-    private boolean acquireLock(final Session session) throws OXException {
+    private static void close(final POP3Folder inbox, boolean doExpunge, POP3Config pop3Config) {
+        try {
+            if (inbox.isOpen()) {
+                inbox.close(doExpunge);
+            }
+        } catch (Exception e) {
+            LOG.warn("POP3 mailbox {} could not be expunged/closed for login {}", pop3Config.getServer(), pop3Config.getLogin(), e);
+        }
+    }
+
+    private DatabaseLock acquireLock(final Session session) throws OXException {
         // Acquire DB lock
-        final int cid = session.getContextId();
-        final Connection con = Database.get(cid, true);
+        int contextId = session.getContextId();
         PreparedStatement stmt = null;
         ResultSet rs = null;
+        final Connection con = Database.get(contextId, true);
         try {
-            final int user = session.getUserId();
+            final int userId = session.getUserId();
             final int accountId = pop3Access.getAccountId();
             int pos = 1;
-            stmt = con.prepareStatement("SELECT 1 FROM user_mail_account_properties WHERE id=? AND cid=? AND user=? AND name=?");
+            stmt = con.prepareStatement("SELECT `value` FROM `user_mail_account_properties` WHERE `id`=? AND `cid`=? AND `user`=? AND `name`=?");
             stmt.setInt(pos++, accountId);
-            stmt.setInt(pos++, cid);
-            stmt.setInt(pos++, user);
+            stmt.setInt(pos++, contextId);
+            stmt.setInt(pos++, userId);
             stmt.setString(pos++, "pop3.lock");
             rs = stmt.executeQuery();
-            final boolean update = rs.next();
+            long stamp = rs.next() ? Long.parseLong(rs.getString(1)) : 0L;
             closeSQLStuff(rs, stmt);
             rs = null;
             stmt = null;
+
+            long now = System.currentTimeMillis();
+            int rows;
             pos = 1;
-            // Either INSERT or UPDATE
-            if (update) {
-                stmt = con.prepareStatement("UPDATE user_mail_account_properties SET value=? WHERE id=? AND cid=? AND user=? AND name=? AND value=?");
-                stmt.setString(pos++, "1");
+            if (stamp == 0) {
+                // No lock entry
+                stmt = con.prepareStatement("INSERT INTO `user_mail_account_properties` (`id`,`cid`,`user`,`name`,`value`) VALUES (?,?,?,?,?) ON DUPLICATE KEY UPDATE `cid`=`cid`");
                 stmt.setInt(pos++, accountId);
-                stmt.setInt(pos++, cid);
-                stmt.setInt(pos++, user);
+                stmt.setInt(pos++, contextId);
+                stmt.setInt(pos++, userId);
                 stmt.setString(pos++, "pop3.lock");
-                stmt.setString(pos++, "0");
-                final int result = stmt.executeUpdate();
-                if (result <= 0) {
-                    // Another thread is already in process.
-                    return false;
-                }
+                stmt.setString(pos++, Long.toString(now));
+                rows = stmt.executeUpdate();
+                closeSQLStuff(stmt);
+                stmt = null;
+            } else if (now - stamp > 20000L) {
+                // Lock entry expired
+                stmt = con.prepareStatement("UPDATE `user_mail_account_properties` SET `value`=? WHERE `id`=? AND `cid`=? AND `user`=? AND `name`=? AND `value`=?");
+                stmt.setString(pos++, Long.toString(now));
+                stmt.setInt(pos++, accountId);
+                stmt.setInt(pos++, contextId);
+                stmt.setInt(pos++, userId);
+                stmt.setString(pos++, "pop3.lock");
+                stmt.setString(pos++, Long.toString(stamp));
+                rows = stmt.executeUpdate();
+                closeSQLStuff(stmt);
+                stmt = null;
             } else {
-                stmt = con.prepareStatement("INSERT INTO user_mail_account_properties (id,cid,user,name,value) VALUES (?,?,?,?,?)");
-                stmt.setInt(pos++, accountId);
-                stmt.setInt(pos++, cid);
-                stmt.setInt(pos++, user);
-                stmt.setString(pos++, "pop3.lock");
-                stmt.setString(pos++, "1");
-                // Uniquely INSERT
-                try {
-                    final int result = stmt.executeUpdate();
-                    if (result <= 0) {
-                        // Another thread is already in process.
-                        return false;
+                // Still locked...
+                rows = 0;
+            }
+
+            if (rows <= 0) {
+                return null;
+            }
+
+            ScheduledTimerTask timerTask = null;
+            boolean error = true;
+            try {
+                TimerService timerService = POP3ServiceRegistry.getServiceRegistry().getOptionalService(TimerService.class);
+                if (timerService == null) {
+                    throw ServiceExceptionCode.absentService(TimerService.class);
+                }
+
+                Runnable task = () -> updateLock(accountId, userId, contextId);
+                long refreshIntervalMillis = 5000L;
+                timerTask = timerService.scheduleWithFixedDelay(task, refreshIntervalMillis, refreshIntervalMillis, TimeUnit.MILLISECONDS);
+
+                DatabaseLock lock = new DatabaseLock(accountId, userId, contextId, timerTask);
+                error = false;
+                return lock;
+            } finally {
+                if (error) {
+                    deleteLock(accountId, userId, contextId);
+                    if (timerTask != null) { // NOSONARLINT
+                        timerTask.cancel();
                     }
-                } catch (SQLException e) {
-                    // INSERT failed. Another thread is already in process.
-                    return false;
                 }
             }
-            // Locked...
-            return true;
         } catch (SQLException e) {
             // Concurrency check failed
             throw POP3ExceptionCode.SQL_ERROR.create(e, e.getMessage());
         } finally {
             closeSQLStuff(rs, stmt);
-            Database.back(cid, true, con);
+            Database.back(contextId, true, con);
         }
     }
 
-    private void releaseLock(final Session session) throws OXException {
-        // Release DB lock
-        final int cid = session.getContextId();
-        final Connection con = Database.get(cid, true);
+    private static void deleteLock(int accountId, int userId, int contextId) {
+        Connection con = null;
         PreparedStatement stmt = null;
         try {
+            con = Database.get(contextId, true);
+            stmt = con.prepareStatement("DELETE FROM `user_mail_account_properties` WHERE `id`=? AND `cid`=? AND `user`=? AND `name`=?");
+            stmt.setInt(1, accountId);
+            stmt.setInt(2, contextId);
+            stmt.setInt(3, userId);
+            stmt.setString(4, "pop3.lock");
+            stmt.executeUpdate();
+        } catch (Exception e) {
+            LOG.warn("Failed to delete POP3 processing lock for account {} of user {} in context {}", I(accountId), I(userId), I(contextId), e);
+        } finally {
+            closeSQLStuff(stmt);
+            Database.back(contextId, true, con);
+        }
+    }
+
+    private static void updateLock(int accountId, int userId, int contextId) {
+        PreparedStatement stmt = null;
+        Connection con = null;
+        try {
+            con = Database.get(contextId, true);
             int pos = 1;
-            stmt = con.prepareStatement("UPDATE user_mail_account_properties SET value=? WHERE id=? AND cid=? AND user=? AND name=?");
-            stmt.setString(pos++, "0");
-            stmt.setInt(pos++, pop3Access.getAccountId());
-            stmt.setInt(pos++, cid);
-            stmt.setInt(pos++, session.getUserId());
+            stmt = con.prepareStatement("UPDATE `user_mail_account_properties` SET `value`=? WHERE `id`=? AND `cid`=? AND `user`=? AND `name`=?");
+            stmt.setString(pos++, Long.toString(System.currentTimeMillis()));
+            stmt.setInt(pos++, accountId);
+            stmt.setInt(pos++, contextId);
+            stmt.setInt(pos++, userId);
             stmt.setString(pos++, "pop3.lock");
             stmt.executeUpdate();
-        } catch (SQLException e) {
-            // Concurrency check failed
-            throw POP3ExceptionCode.SQL_ERROR.create(e, e.getMessage());
+        } catch (Exception e) {
+            LOG.warn("Failed to update POP3 processing lock for account {} of user {} in context {}", I(accountId), I(userId), I(contextId), e);
         } finally {
-            closeSQLStuff(null, stmt);
-            Database.back(cid, true, con);
+            closeSQLStuff(stmt);
+            if (con != null) {
+                Database.back(contextId, true, con);
+            }
         }
     }
 
@@ -789,23 +843,24 @@ public class MailAccountPOP3Storage implements POP3Storage, IMailStoreAware {
      */
     public boolean isDeleteWriteThrough() throws OXException {
         final String property = properties.getProperty(POP3StoragePropertyNames.PROPERTY_DELETE_WRITE_THROUGH);
-        return null == property ? false : Boolean.parseBoolean(property.trim());
+        return null != property && Boolean.parseBoolean(property.trim());
     }
 
-    private int add2Storage(final POP3Folder inbox, final int start, final int len, final boolean uidlNotSupported, final int messageCount, final TIntObjectMap<String> seqnum2uidl) throws MessagingException, OXException {
-        final int retval; // The number of messages added to storage
+    private Add2StorageResult add2Storage(final POP3Folder inbox, final int start, final int len, final boolean uidlNotSupported, final int messageCount, final TIntObjectMap<String> seqnum2uidl) throws MessagingException, OXException {
+        final int numberOfAddedMessages; // The number of messages added to storage
         final int end; // The ending sequence number (inclusive)
         {
             final int startIndex = start - 1;
             final int remaining = messageCount - startIndex;
             if (remaining >= len) {
                 end = startIndex + len;
-                retval = len;
+                numberOfAddedMessages = len;
             } else {
                 end = messageCount;
-                retval = remaining;
+                numberOfAddedMessages = remaining;
             }
         }
+        Set<String> uidlsOfFailedMessages = new HashSet<>();
         /*
          * Check for possible warnings. If so append messages to storage without filtering new messages.
          */
@@ -813,8 +868,8 @@ public class MailAccountPOP3Storage implements POP3Storage, IMailStoreAware {
             /*
              * Append messages to storage
              */
-            doBatchAppendWithFallback(inbox, inbox.getMessages(start, end), seqnum2uidl);
-            return retval;
+            doBatchAppendWithFallback(inbox, inbox.getMessages(start, end), seqnum2uidl, uidlsOfFailedMessages);
+            return new Add2StorageResult(numberOfAddedMessages, uidlsOfFailedMessages.isEmpty() ? null : uidlsOfFailedMessages);
         }
         /*-
          * From JavaDoc for javax.mail.Folder.getMessages():
@@ -828,9 +883,7 @@ public class MailAccountPOP3Storage implements POP3Storage, IMailStoreAware {
          */
         final List<Message> toFetch = new ArrayList<Message>(messages.length);
         if (storageUIDLs.isEmpty()) {
-            for (int i = 0; i < messages.length; i++) {
-                toFetch.add(messages[i]);
-            }
+            toFetch.addAll(Arrays.asList(messages));
         } else {
             for (int i = 0; i < messages.length; i++) {
                 final Message message = messages[i];
@@ -856,9 +909,9 @@ public class MailAccountPOP3Storage implements POP3Storage, IMailStoreAware {
             /*
              * Do batch-append
              */
-            doBatchAppendWithFallback(inbox, toFetch.toArray(new Message[toFetch.size()]), seqnum2uidl);
+            doBatchAppendWithFallback(inbox, toFetch.toArray(new Message[toFetch.size()]), seqnum2uidl, uidlsOfFailedMessages);
         }
-        return retval;
+        return new Add2StorageResult(numberOfAddedMessages, uidlsOfFailedMessages.isEmpty() ? null : uidlsOfFailedMessages);
     }
 
     private static final FetchProfile FETCH_PROFILE_ENVELOPE = new FetchProfile() {
@@ -869,30 +922,34 @@ public class MailAccountPOP3Storage implements POP3Storage, IMailStoreAware {
         }
     };
 
-    private void doBatchAppendWithFallback(final POP3Folder inbox, final Message[] msgs, final TIntObjectMap<String> seqnum2uidl) throws OXException {
+    private void doBatchAppendWithFallback(POP3Folder inbox, Message[] msgs, TIntObjectMap<String> seqnum2uidl, Set<String> uidlsOfFailedMessages) throws OXException {
         final MailAccountPOP3MessageStorage pop3MessageStorage = (MailAccountPOP3MessageStorage) getMessageStorage();
         if (pop3MessageStorage.isMimeSupported()) {
-            try {
-                pop3MessageStorage.appendPOP3Messages(msgs, seqnum2uidl);
-            } catch (OXException e) {
-                LOG.debug("Batch append operation to POP3 storage failed", e);
-                final Throwable cause = e.getCause();
-                if ((cause instanceof MessagingException) && com.openexchange.java.Strings.toLowerCase(cause.getMessage()).indexOf("quota") >= 0) {
-                    /*
-                     * Apparently cause by exceeded quota constraint; abort immediately
-                     */
-                    throw POP3ExceptionCode.QUOTA_CONSTRAINT.create(cause, new Object[0]);
-                }
-                /*
-                 * Retry one-by-one. Handling each mail message.
-                 */
-                final Message[] arr = new Message[1];
-                for (final Message msg : msgs) {
+            OXException quotaExceededException = null;
+            Message[] arr = new Message[1];
+            for (Message msg : msgs) {
+                if (quotaExceededException == null) {
                     try {
                         arr[0] = msg;
                         pop3MessageStorage.appendPOP3Messages(arr, seqnum2uidl);
-                    } catch (OXException inner) {
-                        LOG.warn("POP3 message could not be appended to POP3 storage", inner);
+                    } catch (OXException e) {
+                        if (MimeMailExceptionCode.QUOTA_EXCEEDED.equals(e) || seemsOverQuotaError(e.getCause())) {
+                            /*
+                             * Apparently cause by exceeded quota constraint; abort immediately
+                             */
+                            quotaExceededException = POP3ExceptionCode.QUOTA_CONSTRAINT.create(e.getCause() == null ? e : e.getCause());
+                        } else {
+                            LOG.warn("POP3 message could not be appended to POP3 storage", e);
+                        }
+                        String uidl = seqnum2uidl.get(msg.getMessageNumber());
+                        if (uidl != null) {
+                            uidlsOfFailedMessages.add(uidl);
+                        }
+                    }
+                } else {
+                    String uidl = seqnum2uidl.get(msg.getMessageNumber());
+                    if (uidl != null) {
+                        uidlsOfFailedMessages.add(uidl);
                     }
                 }
             }
@@ -920,9 +977,9 @@ public class MailAccountPOP3Storage implements POP3Storage, IMailStoreAware {
             /*
              * Append them to storage
              */
-            final List<MailMessage> toAppend = new ArrayList<MailMessage>(msgs.length);
+            List<MailMessage> toAppend = new ArrayList<MailMessage>(msgs.length);
             for (int i = 0; i < msgs.length; i++) {
-                final Message message = msgs[i];
+                Message message = msgs[i];
                 if (null != message) {
                     final int msgno = message.getMessageNumber();
                     try {
@@ -934,36 +991,48 @@ public class MailAccountPOP3Storage implements POP3Storage, IMailStoreAware {
                     }
                 }
             }
-            /*
-             * First try batch append operation
-             */
-            try {
-                pop3MessageStorage.appendPOP3Messages(toAppend.toArray(new MailMessage[toAppend.size()]));
-            } catch (OXException e) {
-                LOG.debug("Batch append operation to POP3 storage failed", e);
-                final Throwable cause = e.getCause();
-                if ((cause instanceof MessagingException) && com.openexchange.java.Strings.toLowerCase(cause.getMessage()).indexOf("quota") >= 0) {
-                    /*
-                     * Apparently cause by exceeded quota constraint; abort immediately
-                     */
-                    throw POP3ExceptionCode.QUOTA_CONSTRAINT.create(cause, new Object[0]);
-                }
-                /*
-                 * Retry one-by-one. Handling each mail message.
-                 */
-                final MailMessage[] arr = new MailMessage[1];
-                for (final MailMessage mailMessage : toAppend) {
+            OXException quotaExceededException = null;
+            MailMessage[] arr = new MailMessage[1];
+            for (MailMessage mailMessage : toAppend) {
+                if (quotaExceededException == null) {
                     try {
                         arr[0] = mailMessage;
                         pop3MessageStorage.appendPOP3Messages(arr);
-                    } catch (OXException inner) {
-                        LOG.warn("POP3 message could not be appended to POP3 storage", inner);
+                    } catch (OXException e) {
+                        if (MimeMailExceptionCode.QUOTA_EXCEEDED.equals(e) || seemsOverQuotaError(e.getCause())) {
+                            /*
+                             * Apparently cause by exceeded quota constraint; abort immediately
+                             */
+                            quotaExceededException = POP3ExceptionCode.QUOTA_CONSTRAINT.create(e.getCause() == null ? e : e.getCause());
+                        } else {
+                            LOG.warn("POP3 message could not be appended to POP3 storage", e);
+                        }
+                        String uidl = mailMessage.getMailId();
+                        if (uidl != null) {
+                            uidlsOfFailedMessages.add(uidl);
+                        }
+                    }
+                } else {
+                    String uidl = mailMessage.getMailId();
+                    if (uidl != null) {
+                        uidlsOfFailedMessages.add(uidl);
                     }
                 }
             }
         }
     }
 
+    /**
+     * Checks specified cause if it appears to signal an over-quota error from mail storage.
+     *
+     * @param cause The cause to examine
+     * @return <code>true</code> if considered to be an over-quota error; otherwise <code>false</code>
+     */
+    private static boolean seemsOverQuotaError(Throwable cause) {
+        String detailMsg = Strings.asciiLowerCase(cause.getMessage());
+        return Strings.isNotEmpty(detailMsg) && (detailMsg.indexOf("[overquota]") >= 0 || detailMsg.indexOf("quota") >= 0);
+    }
+
     /**
      * Gets all known UIDLs of the messages kept in this storage.
      *
@@ -990,7 +1059,7 @@ public class MailAccountPOP3Storage implements POP3Storage, IMailStoreAware {
         return Utility.prependPath2Fullname(path, getSeparator(), fullname);
     }
 
-    private void clear(POP3Message[] messageCache) {
+    private static void clear(POP3Message[] messageCache) {
         if (null != messageCache) {
             for (int i = messageCache.length; i-- > 0;) {
                 messageCache[i] = null;
@@ -999,14 +1068,13 @@ public class MailAccountPOP3Storage implements POP3Storage, IMailStoreAware {
     }
 
     private static final Field messageCacheField;
+
     static {
         Field f;
         try {
             f = POP3Folder.class.getDeclaredField("message_cache");
-            f.setAccessible(true);
-        } catch (SecurityException e) {
-            f = null;
-        } catch (NoSuchFieldException e) {
+            f.setAccessible(true); // NOSONARLINT
+        } catch (SecurityException | NoSuchFieldException e) {
             f = null;
         }
         messageCacheField = f;
@@ -1019,14 +1087,55 @@ public class MailAccountPOP3Storage implements POP3Storage, IMailStoreAware {
                 return (POP3Message[]) messageCacheField.get(inbox);
             }
             throw new NoSuchFieldException("message_cache");
-        } catch (SecurityException e) {
-            throw MailExceptionCode.UNEXPECTED_ERROR.create(e, e.getMessage());
-        } catch (IllegalArgumentException e) {
-            throw MailExceptionCode.UNEXPECTED_ERROR.create(e, e.getMessage());
-        } catch (NoSuchFieldException e) {
-            throw MailExceptionCode.UNEXPECTED_ERROR.create(e, e.getMessage());
-        } catch (IllegalAccessException e) {
+        } catch (SecurityException | IllegalArgumentException | NoSuchFieldException | IllegalAccessException e) {
             throw MailExceptionCode.UNEXPECTED_ERROR.create(e, e.getMessage());
         }
     }
+
+    /** Simple helper class to signal number of added messages and the UIDLs of messages that could not be added */
+    private static final class Add2StorageResult {
+
+        /** The number of added messages */
+        final int numberOfAddedMessages;
+
+        /** The UIDLs of messages that could not be added or <code>null</code> */
+        final Set<String> optionalUidlsOfFailedMessages;
+
+        /**
+         * Initializes a new {@link Add2StorageResult}.
+         *
+         * @param numberOfAddedMessages The number of added messages
+         * @param uidlsOfFailedMessages The UIDLs of messages that could not be added
+         */
+        Add2StorageResult(int numberOfAddedMessages, Set<String> optionalUidlsOfFailedMessages) {
+            super();
+            this.numberOfAddedMessages = numberOfAddedMessages;
+            this.optionalUidlsOfFailedMessages = optionalUidlsOfFailedMessages;
+        }
+
+    }
+
+    private static class DatabaseLock {
+
+        private final int accountId;
+        private final int userId;
+        private final int contextId;
+        private final ScheduledTimerTask timerTask;
+
+        DatabaseLock(int accountId, int userId, int contextId, ScheduledTimerTask timerTask) {
+            super();
+            this.accountId = accountId;
+            this.userId = userId;
+            this.contextId = contextId;
+            this.timerTask = timerTask;
+        }
+
+        void unlock() {
+            if (timerTask != null) {
+                timerTask.cancel();
+            }
+            deleteLock(accountId, userId, contextId);
+        }
+    }
+
 }
