@@ -1,5 +1,5 @@
 /*
- * @copyright Copyright (c) OX Software GmbH, Germany <info@open-xchange.com>
+ * @copyright Copyright (c) Open-Xchange GmbH, Germany <info@open-xchange.com>
  * @license AGPL-3.0
  *
  * This code is free software: you can redistribute it and/or modify
@@ -168,7 +168,7 @@ public final class RdbPOP3StorageTrashContainer implements POP3StorageTrashConta
     }
 
     @Override
-    public void addAllUIDL(final Collection<? extends String> uidls) throws OXException {
+    public void addAllUIDL(final Collection<String> uidls) throws OXException {
         Connection con = Database.get(cid, true);
         PreparedStatement stmt = null;
         try {
