@@ -1,5 +1,5 @@
 /*
- * @copyright Copyright (c) OX Software GmbH, Germany <info@open-xchange.com>
+ * @copyright Copyright (c) Open-Xchange GmbH, Germany <info@open-xchange.com>
  * @license AGPL-3.0
  *
  * This code is free software: you can redistribute it and/or modify
@@ -64,80 +64,76 @@ public final class StorageDeleteListener implements MailAccountDeleteListener {
 
     @Override
     public void onBeforeMailAccountDeletion(final int id, final Map<String, Object> eventProps, final int user, final int cid, final Connection con) throws OXException {
-        try {
-            /*
-             * Check if account denotes a POP3 account
-             */
-            final String url;
-            {
-                PreparedStatement stmt = null;
-                ResultSet rs = null;
-                try {
-                    stmt = con.prepareStatement("SELECT url FROM user_mail_account WHERE cid = ? AND id = ? AND user = ?");
-                    stmt.setLong(1, cid);
-                    stmt.setLong(2, id);
-                    stmt.setLong(3, user);
-                    rs = stmt.executeQuery();
-                    if (!rs.next()) {
-                        /*
-                         * Strange...
-                         */
-                        return;
-                    }
-                    url = rs.getString(1).toLowerCase(Locale.ENGLISH);
-                } catch (SQLException e) {
-                    throw MailAccountExceptionCodes.SQL_ERROR.create(e, e.getMessage());
-                } finally {
-                    Databases.closeSQLStuff(rs, stmt);
-                }
-            }
-            if (!url.startsWith(POP3Provider.PROTOCOL_POP3.getName())) {
-                /*
-                 * Not a POP3 account...
-                 */
-                return;
-            }
-            /*
-             * Delete storage content for user if a valid session can be found
-             */
-            SessiondService sessiondService = POP3ServiceRegistry.getServiceRegistry().getService(SessiondService.class);
-            if (null != sessiondService) {
-                boolean dropped = false;
-                for (Session session : sessiondService.getSessions(user, cid)) {
-                    if (!dropped){
-                        final POP3Access pop3Access = POP3Access.newInstance(session, id);
-                        final POP3Storage pop3Storage = pop3Access.getPOP3Storage();
-                        pop3Storage.drop();
-                        dropped = true;
-                    }
-
-                    String key = SessionParameterNames.getStorageProperties(id);
-                    session.setParameter(key, null);
-                    key = SessionParameterNames.getTrashContainer(id);
-                    session.setParameter(key, null);
-                    key = SessionParameterNames.getUIDLMap(cid);
-                    session.setParameter(key, null);
-                }
-            }
-            /*
-             * Drop database entries
-             */
-            final boolean restoreConstraints = disableForeignKeyChecks(con);
+        /*
+         * Check if account denotes a POP3 account
+         */
+        final String url;
+        {
+            PreparedStatement stmt = null;
+            ResultSet rs = null;
             try {
-                RdbPOP3StorageProperties.dropProperties(id, user, cid, con);
-                RdbPOP3StorageTrashContainer.dropTrash(id, user, cid, con);
-                RdbPOP3StorageUIDLMap.dropIDs(id, user, cid, con);
+                stmt = con.prepareStatement("SELECT url FROM user_mail_account WHERE cid = ? AND id = ? AND user = ?");
+                stmt.setLong(1, cid);
+                stmt.setLong(2, id);
+                stmt.setLong(3, user);
+                rs = stmt.executeQuery();
+                if (!rs.next()) {
+                    /*
+                     * Strange...
+                     */
+                    return;
+                }
+                url = rs.getString(1).toLowerCase(Locale.ENGLISH);
+            } catch (SQLException e) {
+                throw MailAccountExceptionCodes.SQL_ERROR.create(e, e.getMessage());
             } finally {
-                if (restoreConstraints) {
-                    try {
-                        enableForeignKeyChecks(con);
-                    } catch (SQLException e) {
-                        org.slf4j.LoggerFactory.getLogger(StorageDeleteListener.class).error("", e);
-                    }
+                Databases.closeSQLStuff(rs, stmt);
+            }
+        }
+        if (!url.startsWith(POP3Provider.PROTOCOL_POP3.getName())) {
+            /*
+             * Not a POP3 account...
+             */
+            return;
+        }
+        /*
+         * Delete storage content for user if a valid session can be found
+         */
+        SessiondService sessiondService = POP3ServiceRegistry.getServiceRegistry().getService(SessiondService.class);
+        if (null != sessiondService) {
+            boolean dropped = false;
+            for (Session session : sessiondService.getSessions(user, cid)) {
+                if (!dropped){
+                    final POP3Access pop3Access = POP3Access.newInstance(session, id);
+                    final POP3Storage pop3Storage = pop3Access.getPOP3Storage();
+                    pop3Storage.drop();
+                    dropped = true;
+                }
+
+                String key = SessionParameterNames.getStorageProperties(id);
+                session.setParameter(key, null);
+                key = SessionParameterNames.getTrashContainer(id);
+                session.setParameter(key, null);
+                key = SessionParameterNames.getUIDLMap(cid);
+                session.setParameter(key, null);
+            }
+        }
+        /*
+         * Drop database entries
+         */
+        final boolean restoreConstraints = disableForeignKeyChecks(con);
+        try {
+            RdbPOP3StorageProperties.dropProperties(id, user, cid, con);
+            RdbPOP3StorageTrashContainer.dropTrash(id, user, cid, con);
+            RdbPOP3StorageUIDLMap.dropIDs(id, user, cid, con);
+        } finally {
+            if (restoreConstraints) {
+                try {
+                    enableForeignKeyChecks(con);
+                } catch (SQLException e) {
+                    org.slf4j.LoggerFactory.getLogger(StorageDeleteListener.class).error("", e);
                 }
             }
-        } catch (OXException e) {
-            throw e;
         }
     }
 
