@@ -1,5 +1,5 @@
 /*
- * @copyright Copyright (c) OX Software GmbH, Germany <info@open-xchange.com>
+ * @copyright Copyright (c) Open-Xchange GmbH, Germany <info@open-xchange.com>
  * @license AGPL-3.0
  *
  * This code is free software: you can redistribute it and/or modify
@@ -41,6 +41,7 @@ import com.openexchange.exception.OXException;
 import com.openexchange.java.Strings;
 import com.openexchange.mail.mime.MimeDefaultSession;
 import com.openexchange.net.ssl.SSLSocketFactoryProvider;
+import com.openexchange.net.ssl.config.SSLConfigurationService;
 import com.openexchange.pop3.POP3ExceptionCode;
 import com.openexchange.pop3.config.IPOP3Properties;
 import com.openexchange.pop3.services.POP3ServiceRegistry;
@@ -512,6 +513,11 @@ public final class POP3CapabilityCache {
             String sslProtocols = pop3Properties.getSSLProtocols();
             if (Strings.isNotEmpty(sslProtocols)) {
                 pop3Props.put("mail.pop3.ssl.protocols", sslProtocols);
+            } else {
+                SSLConfigurationService sslConfigService = POP3ServiceRegistry.getServiceRegistry().getService(SSLConfigurationService.class);
+                if (sslConfigService != null) {
+                    pop3Props.put("mail.pop3.ssl.protocols", Strings.toWhitespaceSeparatedList(sslConfigService.getSupportedProtocols()));
+                }
             }
         }
 
@@ -519,6 +525,11 @@ public final class POP3CapabilityCache {
             String cipherSuites = pop3Properties.getSSLCipherSuites();
             if (Strings.isNotEmpty(cipherSuites)) {
                 pop3Props.put("mail.pop3.ssl.ciphersuites", cipherSuites);
+            } else {
+                SSLConfigurationService sslConfigService = POP3ServiceRegistry.getServiceRegistry().getService(SSLConfigurationService.class);
+                if (sslConfigService != null) {
+                    pop3Props.put("mail.pop3.ssl.ciphersuites", Strings.toWhitespaceSeparatedList(sslConfigService.getSupportedCipherSuites()));
+                }
             }
         }
     }
