@@ -1,5 +1,5 @@
 /*
- * @copyright Copyright (c) OX Software GmbH, Germany <info@open-xchange.com>
+ * @copyright Copyright (c) Open-Xchange GmbH, Germany <info@open-xchange.com>
  * @license AGPL-3.0
  *
  * This code is free software: you can redistribute it and/or modify
@@ -28,6 +28,7 @@ import java.sql.ResultSet;
 import java.sql.SQLException;
 import java.util.ArrayList;
 import java.util.Arrays;
+import java.util.Collections;
 import java.util.HashSet;
 import java.util.List;
 import java.util.Set;
@@ -64,14 +65,9 @@ public class POP3StorageUtil {
      * @throws OXException If POP3 storage provider name cannot be returned
      */
     public static String getPOP3StorageProviderName(final int accountId, final int user, final int cid) throws OXException {
-        final Connection con;
-        try {
-            con = Database.get(cid, false);
-        } catch (OXException e) {
-            throw e;
-        }
         PreparedStatement stmt = null;
         ResultSet rs = null;
+        final Connection con = Database.get(cid, false);
         try {
             stmt = con.prepareStatement(SQL_SELECT_STORAGE_NAME);
             int pos = 1;
@@ -106,13 +102,8 @@ public class POP3StorageUtil {
      * @throws OXException If POP3 storage provider name cannot be set
      */
     public static void setPOP3StorageProviderName(final int accountId, final int user, final int cid, final String name) throws OXException {
-        final Connection con;
-        try {
-            con = Database.get(cid, true);
-        } catch (OXException e) {
-            throw e;
-        }
         PreparedStatement stmt = null;
+        final Connection con = Database.get(cid, true);
         try {
             stmt = con.prepareStatement(SQL_DELETE_STORAGE_NAME);
             int pos = 1;
@@ -177,8 +168,6 @@ public class POP3StorageUtil {
         return removedUIDLs.size();
     }
 
-    private static final String SQL_SELECT_UIDLS = "SELECT uidl FROM pop3_storage_ids WHERE cid = ? AND user = ?";
-
     /**
      * Gets the UIDLs of the messages currently kept in database.
      *
@@ -188,31 +177,30 @@ public class POP3StorageUtil {
      * @throws OXException If UIDLs cannot be retrieved from database
      */
     public static List<String> getUIDLs(final int user, final int cid) throws OXException {
-        final Connection con;
-        try {
-            con = Database.get(cid, false);
-        } catch (OXException e) {
-            throw e;
-        }
-        final List<String> uidls = new ArrayList<String>();
         PreparedStatement stmt = null;
         ResultSet rs = null;
+        final Connection con = Database.get(cid, false);
         try {
-            stmt = con.prepareStatement(SQL_SELECT_UIDLS);
+            stmt = con.prepareStatement("SELECT uidl FROM pop3_storage_ids WHERE cid = ? AND user = ?");
             int pos = 1;
             stmt.setInt(pos++, cid);
             stmt.setInt(pos++, user);
             rs = stmt.executeQuery();
-            while (rs.next()) {
-                uidls.add(rs.getString(1));
+            if (!rs.next()) {
+                return Collections.emptyList();
             }
+
+            List<String> uidls = new ArrayList<String>();
+            do {
+                uidls.add(rs.getString(1));
+            } while (rs.next());
+            return uidls;
         } catch (SQLException e) {
             throw POP3ExceptionCode.SQL_ERROR.create(e, e.getMessage());
         } finally {
             closeSQLStuff(rs, stmt);
             Database.back(cid, false, con);
         }
-        return uidls;
     }
 
     private static final String LOGIN_DELAY = "LOGIN-DELAY";
